/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package beaconconfig;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

class Controls extends JPanel 
    implements ActionListener
{
    private DrawingArea drawArea;
    private SolverControl solver;

    private static JButton reset;
    protected JCheckBox manualOverride;
    protected JCheckBox aggressive;

    protected JCheckBox grid;
    protected JCheckBox annulus;
    protected JCheckBox streamer;

    //JButton addBeacon;

    public Controls(DrawingArea d, SolverControl s)
    {
	setLayout(new GridLayout(2, 3));
	((GridLayout)getLayout()).setHgap(10);

	drawArea = d;
	solver   = s;
	reset = addButton("Reset");
	manualOverride = createCheckBox("Override", false, false);
	aggressive     = createCheckBox("Aggressive", false, false);

	grid     = createCheckBox("Grid", false, true);
	annulus  = createCheckBox("Annulus", false, false);
	streamer = createCheckBox("Streamer", false, false);
    }

    public static JButton getResetButton()
    {
	return reset;
    }

    private JButton addButton(String name)
    {
	JButton result = new JButton(name);
	result.setHorizontalAlignment(JButton.CENTER);
	result.setActionCommand(result.getText());
	result.addActionListener(this);
	add(result);	
	return result;
    }

    public JCheckBox createCheckBox(String label, boolean check, boolean enable)
    {
	JCheckBox result = new JCheckBox(label, check);
	result.setEnabled(enable);
	result.addActionListener(this);
	add(result);
	return result;
    }

    public void enableManualOverride(boolean v)
    {
	if(v != manualOverride.isEnabled())
	    manualOverride.setEnabled(v);
    }

    public void enableTrackingOptions(boolean v)
    {
	if(v != aggressive.isEnabled())
	    aggressive.setEnabled(v);
	if(v != annulus.isEnabled())
	    annulus.setEnabled(v);
	if(v != streamer.isEnabled())
	    streamer.setEnabled(v);
    }

    public void actionPerformed(ActionEvent e) 
    {
	Util.dbgPrintln("Controls::actionPerformed "+e);

	if(e.getActionCommand().equals(reset.getText())) {
	    resetEverything();
	    if(solver != null)
		solver.reset();
	}
	else if(e.getActionCommand().equals(grid.getText())) {
	    if(e.getSource() == null) { // event is from remote
		grid.setSelected(!grid.isSelected());
	    }
	    if(drawArea != null) {
		drawArea.setGrid(grid.isSelected());
	    }
	}
	else if(e.getActionCommand().equals(manualOverride.getText())) {
	    if(e.getSource() == null) // event is from remote
		manualOverride.setSelected(!manualOverride.isSelected());
	    if(solver != null && manualOverride.isSelected()) {
		//Util.dbgPrintln("force tracking!!!!");
		solver.forceTrackingMode();
	    }
	    if(drawArea != null)
		drawArea.setManualOverride(manualOverride.isSelected());
	}
	else if(e.getActionCommand().equals(annulus.getText())) {
	    if(e.getSource() == null) // event is from remote
		annulus.setSelected(!annulus.isSelected());

	    if(drawArea != null)
		drawArea.setAnnulus(annulus.isSelected());

	}
	else if(e.getActionCommand().equals(aggressive.getText())) {
	    if(e.getSource() == null) // event is from remote
		aggressive.setSelected(!aggressive.isSelected());

	    if(solver != null)
		solver.setAggressive(aggressive.isSelected());
	}
	else if(e.getActionCommand().equals(streamer.getText())) {
	    if(e.getSource() == null) // event is from remote
		streamer.setSelected(!streamer.isSelected());

	    if(drawArea != null)
		drawArea.setStreamer(streamer.isSelected());

	}
    }

    public void resetEverything()
    {
	if(drawArea != null) {
	    drawArea.initialize();
	    drawArea.setGrid(false);
	    drawArea.setAnnulus(false);
	    drawArea.setStreamer(false);
	    drawArea.setManualOverride(false);
	}

	manualOverride.setSelected(false);
	aggressive.setSelected(false);
	grid.setSelected(false);	
	annulus.setSelected(false);
	streamer.setSelected(false);

	grid.setEnabled(true);	
	enableManualOverride(false);
	enableTrackingOptions(false);
    }

    public ArrayList getStates()
    {
	ArrayList result = new ArrayList();
	result.add(new Boolean(manualOverride.isSelected()));
	result.add(new Boolean(manualOverride.isEnabled()));
	result.add(new Boolean(aggressive.isSelected()));
	result.add(new Boolean(aggressive.isEnabled()));
	result.add(new Boolean(grid.isSelected()));
	result.add(new Boolean(grid.isEnabled()));
	result.add(new Boolean(annulus.isSelected()));
	result.add(new Boolean(annulus.isEnabled()));
	result.add(new Boolean(streamer.isSelected()));
	result.add(new Boolean(streamer.isEnabled()));
	return result;
    }
}

